class DrawManager {
    constructor(ctx, img) {
        this.ctx = ctx;
        this.img = img;
        this.ctx.msImageSmoothingEnabled = false;
        this.ctx.webkitImageSmoothingEnabled = false;
        this.ctx.imageSmoothingEnabled = false;
        this.ctx.font = "80px '美咲ゴシック'";
    }
    setAlfa(x) {
        this.ctx.globalAlpha = x;
    }
    drawString(s, x, y, color) {
		this.ctx.fillStyle = color;
		this.ctx.fillText(s, x, y);
    }
    drawRect(x, y, width, height, color) {
        this.ctx.beginPath();
        this.ctx.lineWidth = 1;
        this.ctx.fillStyle = color;
        this.ctx.strokeStyle = color;
        this.ctx.rect(x, y, width, height);
        this.ctx.fill();
    }
    drawClearRect(x, y, width, height, color, alfa) {
        this.ctx.globalAlpha = alfa;
        this.drawRect(x, y, width, height, color);
        this.ctx.globalAlpha = 1.0;
    }
    drawStrokeRect(x, y, width, height, color) {
        this.ctx.beginPath();
        this.ctx.lineWidth = 1;
        this.ctx.fillStyle = color;
        this.ctx.strokeStyle = color;
        this.ctx.strokeRect(x, y, width, height);
    }
    drawLine(x1, y1, x2, y2, color) {
        this.ctx.beginPath();
        this.ctx.lineWidth = 1;
        this.ctx.fillStyle = color;
        this.ctx.strokeStyle = color;
        this.ctx.moveTo(x1, y1);
        this.ctx.lineTo(x2, y2);
        this.ctx.stroke();
    }
    drawTriangle(x1, y1, x2, y2, x3, y3, color) {
        this.ctx.beginPath();
        this.ctx.lineWidth = 1;
        this.ctx.fillStyle = color;
        this.ctx.strokeStyle = color;
        this.ctx.moveTo(x1, y1);
        this.ctx.lineTo(x2, y2);
        this.ctx.lineTo(x3, y3);
        this.ctx.fill();
    }
    drawPoint(x, y, color) {
        this.drawRect(x, y, 1, 1, color);
    }
    drawCircle(x, y, r, color) {
        this.ctx.beginPath();
        this.ctx.fillStyle = color;
        this.ctx.strokeStyle = color;
        this.ctx.arc(x, y, r, 0, Math.PI * 2, false);
        this.ctx.stroke();
    }
    drawImage(key, sx, sy, sw, sh, dx, dy, dw, dh) {
        sx = Math.floor(sx);
        sy = Math.floor(sy);
        sw = Math.floor(sw);
        sh = Math.floor(sh);
        dx = Math.floor(dx);
        dy = Math.floor(dy);
        dw = Math.floor(dw);
        dh = Math.floor(dh);
        this.ctx.msImageSmoothingEnabled = false;
        this.ctx.drawImage(this.img[key], sx, sy, sw, sh, dx, dy, dw, dh);
    }
    drawCip(key, cipSize, cx, cy, x, y) {
        this.ctx.msImageSmoothingEnabled = false;
        this.drawImage(key, cipSize * cx, cipSize * cy, cipSize, cipSize, x, y, cipSize * 10, cipSize * 10);
    }
    drawRotateImage(key, x, y, rad) {
        this.ctx.msImageSmoothingEnabled = false;
        this.ctx.rotate(rad);
        let X, Y;
        X = x * Math.cos(rad) + y * Math.sin(rad);
        Y = -x * Math.sin(rad) + y * Math.cos(rad);
        this.ctx.drawImage(this.img[key], X - this.img[key].width / 2, Y - this.img[key].height / 2);
        this.ctx.rotate(-rad);
    }
}
