class Game {
    constructor(dm, ik, im) {
        this.drawManager = dm;
        this.inputKey = ik;
        this.inputMouse = im;
        this.enemies = new Array();
        this.t = 0;
        this.score = 0;
        this.isGameover = false;
        this.set();
        this.drawManager.drawString(" ", 0, 0, "#000000");
    }
    set() {
        this.player = new Player(0, WINDOW_HEIGHT - 160);
        this.isGameover = false;
        this.enemies = new Array();
        this.score = 0;
    }
    update() {
        if (this.isGameover) {
            if (this.inputKey.isDownToggle(39) || this.inputKey.isDownToggle(37)) {
                this.set();
            }
        } else {
            this.enemies.forEach((e) => {
                e.update();
            });
            this.enemies = this.enemies.filter((e) => {
                if (!e.enable) {
                    this.score++;
                }
                return e.enable;
            });
            this.player.update(this.inputKey);
            this.collision();
            if (this.t % 50 === 0) {
                this.enemies.push(new Enemy(Range(0, WINDOW_WIDTH - 80), 180));
            }
            if (this.player.life.life === 0) {
                this.isGameover = true;
            }
        }
        this.t++;
    }
    collision() {
        this.enemies.forEach((enemy) => {
            if (this.player.x < enemy.x + enemy.size && this.player.x + this.player.size > enemy.x && enemy.y > WINDOW_HEIGHT - this.player.size && enemy.y < WINDOW_HEIGHT) {
                this.player.damage();
                enemy.enable = false;
            }
        });
    }
    draw() {
        if (this.isGameover) {
			this.drawManager.drawRect(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT, "#000000");
			this.drawManager.drawString("GAME OVER", WINDOW_WIDTH/2 - 20*("GAME OVER").length, WINDOW_HEIGHT/2 + 20, "#FFFFFF");
			this.drawManager.drawString("SCORE:" + this.score, WINDOW_WIDTH/2 - 20*("SCORE:"+this.score).length, WINDOW_HEIGHT/2 + 100, "#FFFFFF");
        } else {
            this.drawManager.drawCip("BACK", 64, 0, 0, 0, 0);
            this.enemies.forEach((e) => {
                e.draw(this.drawManager);
            });
            this.player.draw(this.drawManager);
            this.drawManager.drawString("SCORE:" + this.score, WINDOW_WIDTH - 40*("SCORE:"+this.score).length, 80, "#000000");
        }
    }
}

class Player {
    constructor(x, y) {
        this.x = x;
        this.y = y;
        this.t = 0;
        this.dir = 0; //0:← 1:→
        this.maxVX = 16;
        this.vx = 0;
        this.ax = 0;
        this.vy = 0;
        this.ay = 0;
        this.size = 160;
        this.isWalking = false;
        this.walkCount = 0;
        this.life = new Life();
        this.invincibleCount = 0;
    }
    update(pk) {
        this.ax = 0;
        if (pk.isDown(37) && this.invincibleCount < 10) {
            this.dir = 0;
            this.ax = -1;
            this.isWalking = true;
        } else if (pk.isDown(39) && this.invincibleCount < 10) {
            this.dir = 1;
            this.ax = 1;
            this.isWalking = true;
        } else {
            if (this.vx > 0 && this.t % 2 === 0) {
                this.vx--;
            }
            if (this.vx < 0 && this.t % 2 === 0) {
                this.vx++;
            }
            if (Math.abs(this.vx) < 5) {
                this.isWalking = false;
            }
        }
        this.vx += this.ax;
        this.vy += this.ay;
        if (this.vx > this.maxVX) {
            this.vx = this.maxVX;
        }
        if (this.vx < -this.maxVX) {
            this.vx = -this.maxVX;
        }

        this.x += this.vx;
        this.y += this.vy;

        if (this.x < 0) {
            this.x = 0;
        }
        if (this.x + this.size > WINDOW_WIDTH) {
            this.x = WINDOW_WIDTH - this.size;
        }
        this.t++;
        if (this.isWalking) {
            this.walkCount++;
        }
        if (this.invincibleCount > 0) {
            this.invincibleCount--;
        }
    }
    damage() {
        if (this.invincibleCount > 0) {
            return;
        }
        this.vx = (this.dir - 0.5) * (-20);
        this.invincibleCount = 30;
        this.life.takeDamage();
    }
    draw(dm) {
        this.life.draw(dm);
        if (Math.floor(this.invincibleCount / 5) % 2 === 1) {
            return;
        }
        if (this.isWalking) {
            dm.drawCip("POYO", 16, Floor(this.walkCount / 10) % 4, this.dir, this.x, this.y);
        } else {
            dm.drawCip("POYO", 16, 0, this.dir, this.x, this.y);
        }
    }
}

class Enemy {
    constructor(x, y) {
        this.x = x;
        this.y = y;
        this.v = -5;
        this.a = 0.5;
        this.t = 0;
        this.size = 80;
        this.enable = true;
    }
    update() {
        this.v += this.a;
        if (this.v > 20)
            this.v = 20;
        this.y += this.v;
        if (this.y > WINDOW_HEIGHT) {
            this.enable = false;
        }
        this.t++;
    }
    draw(dm) {
        if (this.t < 20) {
            dm.setAlfa(this.t / 20);
        }
        if (this.t < 15) {
            dm.drawCip("ENEMY", 8, 0, 0, this.x, this.y);
        } else {
            dm.drawCip("ENEMY", 8, Math.floor(this.t / 5) % 4, 0, this.x, this.y);
        }
        dm.setAlfa(1);
    }
}

class Life {
    constructor() {
        this.life = 3;
        this.maxLife = 3;
        this.size = 80;
        this.t = 0;
    }
    update() {
        this.t++;
    }
    takeDamage() {
        this.life--;
        if (this.life < 0) {
            this.life = 0;
        }
    }
    draw(dm) {
        for (let i = 0; i < this.maxLife; i++) {
            dm.drawCip("LIFE", 8, (i < this.life ? 0 : 1), 0, i * this.size, 0);
        }
    }
}
